# This file generates the data underlying FIGURE 2 in Hurford et al. This file takes
# several hours to run.

# Clearing Workspace and loading packages
rm(list = ls())
# This package handles the variable length delays
require(PBSddesolve)

# You will need to set the working directory so this code can find the files
# "ParamsFuncs.RData", "farmCH.Rdata", etc
#setwd("/Users/amyhurford/Desktop/Code")

# Fitted functions are loaded from a saved file
load("ParamsFuncs.RData")

#Define the functions for temperature and salinity
Temp <- function(t){
  if(var1 == 1){
    Temp = a + b1*sin(2 * pi * t/365) + b2*cos(2 * pi * t/365)
  }
  else{
    Temp = rep(a1,length(t))
  }
  return(Temp)
}
S <- function(t){
  if(var1==1){
    S = c + d1*sin(2 * pi * t/365) + d2*cos(2 * pi * t/365)
  }
  else{
    S = rep(c,length(t))
  }
  return(S)
}

# Returns the values of the state variables after the numerical integration
# t0, tfin: start/end time of integration
# A0, I0: initial number of adult females (A0) and copepodids (I0)
# pastvals: historical values of the variables, required for delay equations
# lambda: needed to calculate R0. See Section S4 of the SOM.
# g: g multiplies f, the density of fish, and g is used to calculate the critical stocking
# density
sealice=function(t0,tfin,A0,I0,pastvals,lambda,g){
## Function to calculate the gradient, i.e. the functions that will be integrated.
  lice = function(t, y){
    # The number of adult females
    A = y[1]
    # The number of infectious copepodids
    I = y[2]
    # The time delay for the planktonic stage
    tp = y[3]
    # The time delay for the chalimus stage
    tc = y[4]
    # Mortality durign the planktonic stage
    PhiP = y[5]
    # Mortality during the chalimus stage
    PhiC = y[6]
    
    # Apply timelags only once enough history has been established.
    if (t - tp < t0) {
      # Here if t-tp = -1 then we get 366-1 = the 365th element of pastvals which corresposds
      # to Dec 31.
      lagp = pastvals[366+round(t-tp,0),]
    } else{
      # if the logical test passses we can calculate the lags
      lagp = pastvalue(t - tp)
    }
    
    if (t - tc < t0) {
      lagc = pastvals[366+round(t-tc,0),]
    } else{
      # if the logical test passses we can calculate the lags
      lagc = pastvalue(t - tc)
    }
    
    # The system of DDEs: only dI/dt and dA/dt are needed. See Section S4
    I1 =  (eta(Temp(t-tp))*epsilon(Temp(t-tp))*lagp[1]*(gammaP(Temp(t))/gammaP(Temp(t-tp)))*V(Temp(t-tp),S(t-tp))*PhiP)/lambda-iota*g*f*I-mui(S(t))*I
    A1 = (iota*g*f*lagc[2]*(gammaC(Temp(t))/gammaC(Temp(t-tc)))*PhiC)/2/lambda-mua(S(t))*A
    
    tp1 = 1 - gammaP(Temp(t))/gammaP(Temp(t-tp))
    tc1 = 1 - gammaC(Temp(t))/gammaC(Temp(t-tc))
    
    PhiP1 = PhiP*(gammaP(Temp(t))*mup(S(t-tp))/gammaP(Temp(t-tp))-mup(S(t)))
    PhiC1 = PhiC*(gammaC(Temp(t))*muc(S(t-tc))/gammaC(Temp(t-tc))-muc(S(t)))
    return(c(A1, I1, tp1, tc1, PhiP1, PhiC1))
  }
  # List of all paremeter Values
  parameters = c(eta, epsilon, iota, V, mup, mui, muc, mua, t0, tfin, mup, f)
  # Calculates the initial values of the time delays etc.
  tp0 = tpINIT()
  tc0 = tcINIT()
  PhiP0 = exp(-(integrate(mortalityP, t0 - tp0, t0)$value))
  PhiC0 = exp(-(integrate(mortalityC, t0 - tc0, t0)$value))
  # List of all initial Values
  yinit <- c(A0, I0, tp0, tc0, PhiP0, PhiC0);
  # perform the numerical integration
  x = dde(y = yinit, func = lice, parms = parameters, times = seq(t0, tfin, 1), hbsize = 1000000)
  return(x)
}

# Function used to calculate the critical stocking density, R0 and floquet exponent.
SpectralBound = function(lambda,g){
  indx=NULL
  A0=1
  I0=1
  # pastvals is the history for the first iteration of the procedure.
  pastvals = matrix(rep(A0,366), rep(I0,366),ncol = 2,nrow=366)
  for(i in seq(1,100)){
    # Numerically solve the system
    x = sealice(0,365,A0,I0, pastvals,lambda,g)
    # Calculate the norm
    indx[i] = max(c(max(x$y1), max(x$y2)))
    # Normalize the numerical solution for this iteration
    A0= tail(x$y1,n=1)/indx[i]
    I0= tail(x$y2,n=1)/indx[i]
    # Use the normalized numerical solutions for past values on the next iteration
    pastvals = matrix(c(x$y1/indx[i], x$y2/indx[i]), ncol = 2,nrow=366)
    # Break if the value of the norm is converging, otherwise repeat 100 times.
    if(i>1 && (abs(indx[i]-indx[i-1])<1e-5)){
      break
    }
  }
  # We would like to find when the final value of indx is 1, so we subtract one
  # and will use uniroot() to find when r is zero.
  r = indx[i]-1
  return(r)
}

# Preallocation
fish = NULL
Location = NULL
iotaVal = NULL
R0Val = NULL
R0gVal = NULL
FVal = NULL
varb = NULL

# This farm is being referred to as transitional based on Groner et al. 2016.
# These data correspond to the BCB site.
load('farm24.RData')
# Initial conditions
P0<-f
I0<-f
A0<-f
C0<-f
t0<-0
# iotav and iotac set up the iota values for assuming/ignoring seasonality
iotav<-iota
iotac<-iota1
iota<-iotav
# var1<-1 means that seasonality is considered and iota = iotav.
var1<-1
# g is a coefficient that multiplies f. It is set = 1 so that we can
# just calculate R0.
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
# We set lambda = 1 to calculate the floquet exponent.
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
# This adds one to the spectral bound because 1 had been subtracted within
# the function SpectralBound(). This formulate relates the floquet multiplier
# to the Floquet exponent.
FEst = log(SpectralBound(1,1)+1)/365

# Below we are prepping all the output for this site to assemble it in a table for
# all sites.
fish = rbind(fish,unname(f))
Location = rbind(Location,"BCB")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
varb = rbind(varb,var1)

# Now we're going to repeat all the calculations but ignoring seasonal variability.
var1<-0
# This is the value of iota ignoring seasonality
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"BCB")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
varb = rbind(varb,var1)

# Now do all the calculations for another site
load('farmWCVI.RData')
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"WCVI")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"WCVI")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))

load('farmCC.RData')
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"CC")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"CC")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))

load('farmNS.RData')
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"NS")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"NS")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))

load('farmNB.RData')
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"NB")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"NB")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))

load('farmNL.RData')
c<-c1
c<-31
d1<-0
d2<-0
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"NL")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"NL")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))

load('farmCH.RData')
c<-31
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"CH")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"CH")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))

load('farmNIN.RData')
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"NIN")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"NIN")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))

load('farmNLI.RData')
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"NLI")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"NLI")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))

load('farmIM3.RData')
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"IM3")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"IM3")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))

load('farmIM4.RData')
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"IM4")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1, upper=3, g=1)$root
R0gEst = uniroot(SpectralBound, lower=0, upper=1, lambda=1)$root
FEst = log(SpectralBound(1,1)+1)/365
fish = rbind(fish,unname(f))
Location = rbind(Location,"IM4")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,unname(R0gEst))
FVal = rbind(FVal,unname(FEst))

# When we first did the NL site we set salinity to 31 psu. The real value is used below.
# Note that d1 and d2 are set to 0 because there isn't much evidence of seasonality in these
# data
load('farmNL.RData')
c<-c1
d1<-0
d2<-0
var1<-1
iota<-iotav
R0Est = uniroot(SpectralBound, lower=1e-2, upper=1e-1, g=1)$root
FEst = log(SpectralBound(1,1)+1)/365

# This site with the real value of salinity for NL is called NLlow
fish = rbind(fish,unname(f))
Location = rbind(Location,"NLlow")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,100)
FVal = rbind(FVal,unname(FEst))
var1<-0
iota<-iotac
R0Est = uniroot(SpectralBound, lower=1e-4, upper=1e-3, g=1)$root
FEst = log(SpectralBound(1,1)+1)/365

fish = rbind(fish,unname(f))
Location = rbind(Location,"NLlow")
iotaVal = rbind(iotaVal,unname(iota))
R0Val = rbind(R0Val,unname(R0Est))
R0gVal = rbind(R0gVal,100)
FVal = rbind(FVal,unname(FEst))

# The critical stocking density is recovered by multiplying f by g.
fcrit= fish*R0gVal
# All the data is outputed as a table at plotted by Figure2_R0Graphs.R
Summary=data.frame(Location,fish,iotaVal,R0Val,R0gVal,fcrit,FVal)
save(Summary,file="Summary.RData")