# MAKES FIGURE S1 of Hurford et al. Regional differences in sea temperature and
# salinity affects salmon lice population dynamics, stage structure, and management.
# Fits functions to data that parameterize the relationships for the sea lice model.
# The output ParamsFuncs.RData is called by other files.
# Contact: Amy Hurford, ahurford@mun.ca

# Clear working environment
rm(list = ls())
# Set working directory: you will need to set your own directory
setwd("/Users/amyhurford/Desktop/Code")

# 1) EGG VIABILITY, v.
# Hatching success, v, from Johnson and Albright Table 1 at 10C for a range of
# salinities
pHatchJA = c(0,0.7,0.78,1)
SalJA = c(10,15,20,25)
TempJA = c(10,10,10,10)
# Hatching success, v, from Samsing depends on temperature, for 34 psu.
SalS = c(34,34,34,34,34,34)
TempS = c(20,15,10,7,5,3)
pHatchS = c(1,1,.87,.9,.85,.28)
# Bind the data from Samsing and Johnson and Albright into one data frame.
data = data.frame(Temp=TempJA,Sal=SalJA,pHatch=pHatchJA)
data = rbind(data,cbind(Temp=TempS,Sal=SalS,pHatch=pHatchS))
# Vectors of temperature and salinity to plot the fitted functions
Tvec = seq(0, 20, .5)
Svec = seq(0, 35, .5)

# Remove the fifth row: hatching is 100% for both 20 and 15C. 20C is removed.
data=data[-5,]
# Fit model from Samsing
lmod = lm(data$pHatch ~ log(data$Temp)+log(data$Sal))

# Create a function of the fitted values which will be exported for
# use in the code files.
V<-function(T,S){
  V=NULL
  Tvec = T
  Svec=S
  # The function needs to be capable of taking vector arguments for
  # use in other files
  for(i in seq(1,length(Tvec))){
    T = Tvec[i]
    S = Svec[i]
    # The fitted coefficients
  V = rbind(V,-2.2579 + 0.4557*log(T)+0.6373*log(S))
  }
  V[V<0]<-0
  V[V>1]<-1
  return(V)
}

par(mfrow = c(3,4), mar=c(4,4,1,1))
plot(TempS,pHatchS,pch=4, xlim = c(0,20), ylim=c(0,1), main = "Egg viability", xlab = "", ylab="")
lines(Tvec,V(Tvec, rep(34,length(Tvec))))
plot(SalJA,pHatchJA,pch=15,xlim=c(0,35), main="Egg viability", xlab = "", ylab="")
lines(Svec,V(rep(10,length(Svec)),Svec))

######################
# 2) EGGS PER STRING
# This is the fitted function from Samsing
eta = function(Temp){
  exp(5.6 - 0.43*log(Temp/10) - 0.78*log(Temp/10)^2)
}

# Temperature data from the supplementary material of Samsing.
TempS = c(20,15,10,7,5,3)
# Temperature data from Heutsch
TempH = c(7.2,12.2)

plot(Tvec, eta(Tvec), typ = "l", main = "Eggs per string", xlab = "", ylab="")
points(TempS, eta(TempS), pch=4)
points(TempH,c(153*1.06,153),pch=0)

#################
# 2) EGG STRING PRODUCTION RATE, epsilon
# Temperature data from Boxaspen
TempBA = c(2,3,4,5,10)
# Temperature data from Johnson and Albright
TempJA = c(5,10,15)
# Temperature data from Tucker
TempTm = 8.28

# Days to produce an egg string:
# Tucker 
HatchDaysT=10
# Johnson and Albright (corresponding temperature is logged above for eta)
HatchDaysJA=c(17.5,8.6,5.5)
# Boxaspen
HatchDaysBA = c(45.1,35.2,27.6,21.6,8.7)
# Samsing
HatchDaysS = c(1.78, 2.88,4.63,7.64,12.97,20.77)
# Heutch (corresponding temperature is logged above for eta)
HatchDaysH = c(11,5)
# Combine the data for model fitting.
HatchDays = c(HatchDaysS,HatchDaysT,HatchDaysJA,HatchDaysBA,HatchDaysH)
Temp = c(TempS,TempTm,TempJA,TempBA, TempH)
lm.H = lm(log(HatchDays)~log(Temp/10))
b0=unname(coef(lm.H)[1])
b1=unname(coef(lm.H)[2])

epsilon = function(T){
  b0 = 1.86
  b1 = -1.30
  A = exp(b0+b1*log(T/10))
  # Do not extrapolate beyond the longest egg string production time
  A[A>45.1]=45.1
  # The rate is the reciprocal of the time to produce the egg string
  epsilon = 1/A
  return(epsilon)
}

plot(TempS,HatchDaysS,pch=4, ylim = c(0,50), xlim=c(0,20), xlab = "", ylab="", main = "Hatching rate")
points(TempBA,HatchDaysBA, pch=17)
points(TempJA,HatchDaysJA,pch=15)
points(TempH,HatchDaysH,pch=0)
points(TempTm,HatchDaysT,pch=19)
lines(Tvec,1/epsilon(Tvec))
############
#4) MATURATION RATE IN THE NAUPLII STAGE, gammaP
# The duration of the nauplii stage as estimated by Samsing
# Data at 5,7,10,15,20C
gammaPS = function(T){
  NapD = exp(1.4 -1.48*log(T/10))
  mup = 1/NapD
  return(mup)
}

# Tucker data
NapDT = 3
TempTm = 8.28
# Johannessen
TempJ = c(9.2,11,19)
NapDJ = c((35+42)/24, 63/24, 33/24)
# Wooten
TempW = 12
NapDW = 44/24
# Boxaspen
NapDBA = c(23.5,18,14,9.5,4)
# Johnson and Albright
NapDJA = c(222.3/24, 87.4/24,44.8/24)
# Samsing - based on fit
TempS = c(5,7,10,15)
NapDS =  1/gammaPS(TempS)
# Aggregate data
NapD = c(NapDT, NapDJ, NapDW, NapDBA, NapDJA, NapDS)
Temp = c(TempTm, TempJ, TempW, TempBA, TempJA, TempS)
# Fit model
lm.H = lm(log(NapD)~log(Temp/10))
b0=unname(coef(lm.H)[1])
b1=unname(coef(lm.H)[2])

# The fitted function
gammaP = function(T){
  b0 = 1.216
  b1 = -1.38
  A = exp(b0+b1*log(T/10))
  A[A>23.5]=23.5
  gammaP = 1/A
  return(gammaP)
}

# An alternative version of the function that takes t as an argument
# (needed for other files)
gamP = function(t){
  gammaP(Temp(t))
}

plot(Tvec,1/gammaP(Tvec),typ="l", ylim=c(0,50),main="Duration of stage", xlab = "", ylab="")
points(TempJA,NapDJA, pch=15)
points(TempS, NapDS, pch=4)
points(TempTm,NapDT, pch=19)
# Johannessen
points(TempJ, NapDJ)
# Wooten
points(TempW,NapDW, pch=2)
# Boxaspen
points(TempBA, NapDBA, pch=17)

#####################################
# 6) THE DURATION OF THE CHALIMUS STAGE, gammaC
## chalimus delay: a Belahedrek function determines the length of time that
## it takes a chalimus to mature into an adult see louse. This parameterization
## is from Rittenhouse based on Stein et al 2005 Figure 4 and Table 3 for females. The lowest
## temperature considered is 8C so we do not predict higher delays than this.
gammaC = function(Temp){
  Tref = 10
  alphac = 11.94
  Betac = 0.177
  gC = ((Betac * (Temp - Tref + alphac)) / (alphac)) ^ 2
  gC[gC<((Betac * (8 - Tref + alphac)) / (alphac)) ^ 2]=((Betac * (8 - Tref + alphac)) / (alphac)) ^ 2
  return(gC) 
}
# Alternative version of the function that takes time as an argument
gamC = function(t){
  gammaC(Temp(t))
}

lines(Tvec, 1/gammaC(Tvec), typ = "l", main = "Duration of C", ylab = "1/gamma_C", xlab = "Temperature, C", lwd = 1)
# See Figure 4c of Stein for the data underlying this relationship.


################### RELATIONSHIPS WITH SALINITY
# 7) NAUPLII SURVIVAL, mu_P.
# Nauplii survival data is from Table 3 in Johnson and Albright.
# The data point for 34ppt is from Samsing who observed 100%
# progression of nauplii to copepodids at 34ppt (see beginning
# of resutls section in Johnson and Albright)
Psurv = c(0.005/19.8,0.005/51.1, 35.2/65.9, 26.8/54.6,1)
# At 10C the length of the maturation delay for nauplii is 1/gammaP(10)
# with some algebra we find that:
muPvec=-log(Psurv)*gammaP(10)
# The survival probabilities for <25 psu are very close to zero.
# We take the mean of the observations for 20 and 25 psu and use this
# as the minimum survival probability for <25 psu
muPvec2 = c(mean(muPvec[1:2]), muPvec[3:5])
Sal = c(20, 25, 30,30,34)
# The survival probability of 1 is omitted from the fit.
# The value of 25psu is used for the mean probability of
# survivial for <25psu.
mod = lm(log(muPvec2[1:3])~Sal[2:4])
# Function must take vector arguments for use in other files.
mup <- function(S) {
  Svec = S
  mup = rep(0,length(Svec))
  for (i in seq(1,length(Svec))){
    S = Svec[i]
  if(S<=25){
    mup[i] = 2.16
  }
  else if(S>25){
    mup[i] = exp(13.64-0.515*S)
  }
  }
  return(mup)
}
plot(Svec,1/mup(Svec), typ = "l", main = "Days survived by P", ylim = c(0,20), xlab = "", ylab="")
points(Sal,1/muPvec, pch=15)
##########
# 8) COPEPODID SURVIVAL, muI
# Infestation success for Samsing.
Temp = c(10,20,5)
TempT = c(7.5, 8.76, 8.34, 8.53)
# Copepodid survival data from Johnson and Albright
SurvTimeJA = c(2.2,3.2,8,3.9,2.4,3.4,3.4,4.6,3,6.6,6,5.6)
SalJA = c(15, 20, 25,30,15,20,25,30,15,20,25,30)
TempJA = c(10,10,10,10,5,5,5,5,15,15,15,15)
data = data.frame(Temp=TempJA,Sal=SalJA,SurvTime=SurvTimeJA)
# This function was fit by Samsing for copepodid infectious window
mort = function(T){
  lnmort = b0 + b1*log(T/10)+b2*log(T/10)^2
  mort = exp(lnmort)
  return(mort)
}
# These are the parameters reported by Samsing for 34ppt.
# The function is used to recreate the data
b0<-2.4
b1<- -0.26
b2<- -1.03
TempS = c(5,7,10,15,20)
SurvTimeS=mort(c(5,7,10,15,20))
SalS = c(34,34,34,34,34)
# The Samsing data is appended to the data frame
data = rbind(data,cbind(Temp=TempS,Sal=SalS,SurvTime=SurvTimeS))
# Copepod mortality from Tucker et al 2002
SalT = c(33.19, 32.83, 33.27, 33.18)
SurvTimeT = c(1/.181, 1/.061,1/.101, 1/.131)
# The Tucker data is appended to the data frame
data = rbind(data,cbind(Temp=TempT,Sal=SalT,SurvTime=SurvTimeT))
# Data from Bricknell: Time until 50% cohort has died. These data
# report that 50% of the cohort has died in < 1 day. They are not
# consistent with the data from Samsing, Johnson and Albright and
# Tucker and have been ommitted from the fit.
SurvTimeB = c(0,4,6,8,11,24,22,25)/24
SalB = c(12, 16, 19,13,16,29,33,36)
# Model fit
mod1 = lm(log(data$SurvTime)~data$Sal+data$Temp)
# Exporting the model fit as a function for use in other files
mui = function(S){
  exp(-.0232-.059*S)
}
# Plotting the fit and the data
plot(Svec, 1/mui(Svec), , typ = "l", main = "Days survived by I", ylim = c(0,20), xlab = "", ylab="")
points(SalS, SurvTimeS, pch=4)
points(SalT, SurvTimeT, pch=19)
points(SalJA, SurvTimeJA, pch=15)
points(SalB, SurvTimeB, pch=5)
#################################################
# 9) PARASITE SURVIVAL, mua = muc
# Adult mortality as a function of salinity - based on Connors 2008
mua = function(Sal){
  sigma0a = 4.12
  sigma1a = 0.124
  # divide by 24 because Connors is in units of hours
  mua = exp(((sigma0a + sigma1a * Sal)))/24
  mua[mua>1963/24]=1963/24
  fmua = 1/mua
  return(fmua)
}

plot(Svec, 1/mua(Svec),typ = "l", main = "Days survived by C or A", ylim = c(0,80), xlab = "", ylab="" )
points(seq(0,28,7),1/mua(seq(0,28,7)), pch=6)

# chalimus mortality is similar to adult so we use the same eqs
muc = function(Sal){
  muc = mua(Sal)
  return(muc)
}

# mortality as a function of time so that integration can determine
# the probability, phic
mortalityC = function(t){
  Salinity = S(t)
  Mortality = muc(Salinity)
  return(Mortality)
}

# mortality as a function of time so that integration can determine
# the probability, phip
mortalityP = function(t){
  Salinity = S(t)
  Mortality = mup(Salinity)
  return(Mortality)
}

###############################
# 10) INFESTATION SUCCESS, iota1
Pattach = c(16/100,13.3/100,0.62/100)
PattachT = c(.104, .031, .166, .07)
plot(Temp,Pattach, main = "Infestation probability", pch=4, xlim = c(0,20),ylim = c(0,.2), xlab = "", ylab="")
points(TempT,PattachT, pch=19)

## FUNCTIONS GIVING THE INITAL VALUES OF TIME DELAYS
# The Initial Delays (Are "u", where u is such that the integral of gamma from
# t0-u to t0 is equal to 1)
tpINIT = function(){
  IgamP = function(u){
    gP=integrate(gamP, t0 - u, t0)$value-1
    return(gP)
  }
  out=uniroot(IgamP,lower=0.1, upper=200)$root
  return(out)
}

tcINIT = function(){
  IgamC = function(u){ 
    gC=integrate(gamC, t0 - u, t0)$value-1
    return(gC)
  }
  out=uniroot(IgamC,lower=0.1, upper=200)$root
  return(out)
}

## FUNCTIONS FOR THE TEMPERATURE AND SALINITY
Temp = function(t){
  Temp = a + b1*sin(2 * pi * t/365) + b2*cos(2 * pi * t/365)
  return(Temp)
}
S = function(t){
  S = c + d1*sin(2 * pi * t/365) + d2*cos(2 * pi * t/365)
  return(S)
}

# Save all the parameterization functions for use in other files
save(Temp, S, V, eta, epsilon, gammaC, gammaP,gamC, gamP,mup, mui, muc, mua, tpINIT, tcINIT,mortalityC,mortalityP,file="ParamsFuncs.RData")

# Load data from farm 24 (BCB). Show a plot of the best fit iota.
# x uses the fitted emprical data, xc assumes no variation in temperature.
load('farm24.Rdata')
par(las=3, cex.lab=1.2, cex.axis=1.2)
plot(AdultFemalesData$femalesTime, AdultFemalesData$femalesValue, pch=8,ylab = "A/f",xlab="",xlim = c(t0, tfin), main = "Fitted iota (adult females)")
lines(x$time, x$y1/f, lwd=1)
lines(xc$time, xc$y1/f, lty=2, lwd=1)
points(AdultFemalesData$femalesTime, AdultFemalesData$femalesValue, pch=8)

plot(AttachedData$attachedTime, AttachedData$attachedValue, pch=8,xlab = "",ylab = "(2A+C)/f",xlim = c(t0, tfin), main="Fitted iota (parasites)")
lines(x$time, (2*x$y1+x$y5)/f, typ= "l", xaxt = "n", lwd=1)
lines(xc$time, (2*xc$y1+x$y5)/f, lty=2, xaxt = "n", lwd=1)

# Stage structure value for BCB as quoted in text. There is an extra
# data point for females with no corresponding data for chalimii/pre-adults
# so that is removed.
1-2*mean(AdultFemalesData$femalesValue[-7]/AttachedData$attachedValue)
